opus

package
v2.8.3 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 12, 2026 License: MIT Imports: 1 Imported by: 13

Documentation

Overview

Package opus contains utilities to work with the Opus codec.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func PacketDuration deprecated

func PacketDuration(pkt []byte) time.Duration

PacketDuration returns the duration of an Opus packet.

Deprecated: replaced by PacketDuration2

func PacketDuration2 added in v2.1.0

func PacketDuration2(pkt []byte) int64

PacketDuration2 returns the duration of an Opus packet, in 1/48000 seconds. Specification: RFC6716, section 3.1

Types

This section is empty.

Jump to